import os

import api
import embedding
import openai
import pdf
from openai.embeddings_utils import cosine_similarity, get_embedding


if __name__ == '__main__':
    arxiv_id = '2005.14165v4'  # GPT-3
    arxiv_id = '2303.08774v3'  # GPT-4
    openai.api_key = os.environ['OPENAI_API_KEY']

    metadata = api.fetch(arxiv_id)
    file_name = pdf.download(arxiv_id)
    context = pdf.context(file_name)
    chunks = embedding.chunks(context, metadata)
    embeddings = embedding.encode(chunks)
    print([(x[embedding]) for x in embeddings])
    

    # print(f'Number of chunks: {len(chunks)}')
    # print(f'Number of embeddings: {len(embeddings)}')
    # print(chunks[:3])

    # # db = embedding.vector_database(chunks, OpenAIEmbeddings())
    # # print(db.head())

    # # chain = load_qa_chain(OpenAI(temperature=0.0), chain_type="stuff")
    # db = FAISS.from_documents(chunks, OpenAIEmbeddings())
    # qa = ConversationalRetrievalChain.from_llm(OpenAI(temperature=0.0), db.as_retriever())
    # chat_history = []

    # while True:
    #     question = input("Enter a question: ")
    #     if question == "exit()":
    #         break
    #     # docs = db.similarity_search(question)
    #     # answers = chain.run(input_documents=docs, question=question)
    #     # print(answers)

    #     response = qa({"question": question, "chat_history": chat_history})
    #     chat_history.append((question, response['answer']))
    #     print(f'Q: {question}')
    #     print(f'A: {response["answer"]}')

    # question_embedding = embedding.encode_sentence(question)
    # db["similarity"] = db.embedding.apply(lambda x: cosine_similarity(x, question_embedding))
    # ranks = db.sort_values(by="similarity", ascending=False)
    # print(ranks.head(n=1))

    # question_embedding = embedding.encode_sentence(question)
    # db["similarity"] = db.embedding.apply(lambda x: cosine_similarity(x, question_embedding))
    # ranks = db.sort_values(by="similarity", ascending=False)
    # print(ranks.head(n=2))

    # data = response.json()
    # items = data.get("message", {}).get("items", [])
    # if items:
    #     first_item = items[0]
    #     doi = first_item.get("DOI")
    #     if doi:
    #         print(f'Found DOI: {doi}')
    #         doi_url = f"https://doi.org/{doi}"
    #         print(f"DOI URL: {doi_url}")

    # r = requests.get('http://api.crossref.org/works?query.bibliographic="Tyler B. Smith, Momentum-space Gravity from the Quantum Geometry and Entropy of Bloch Electrons"')
    # print(r.content)



import openai
import tiktoken


def token_length(text: str) -> int:
    tokenizer = tiktoken.get_encoding("cl100k_base")
    tokens = tokenizer.encode(
        text,
        disallowed_special=()
    )
    return len(tokens)


def split_text(text, chunk_size, overlap_size):
    chunks = []
    num_chunks = len(text)//(chunk_size-overlap_size)
    for i in range(num_chunks):
        start = i * (chunk_size - overlap_size)
        end = start + chunk_size
        chunk = text[start:end]
        chunks.append(chunk)

    return chunks


def chunks(context: dict, metadata: dict) -> list:
    results = []
    results.append(context['toc'])
    for page, text in enumerate(context['fulltext']):
        chunks = split_text(text, 1024, 32)
        for chunk in chunks:
            chunks_with_metadata = f"[PDF-PAGE]: {page};\t[PAPER-TITLE]: {metadata['title']};\t[PAPER-CONTENT]: {chunk};"
            results.append(chunks_with_metadata)

    return results


def encode(chunks: list) -> list:
    response = openai.Embedding.create(
        input=chunks,
        model="text-embedding-ada-002",
    )

    return response['data']




import os
import fitz
from google.cloud import storage


def download(arxiv_id) -> str:
    os.makedirs("dataset/tmp", exist_ok=True)
    file_name = f"dataset/tmp/{arxiv_id}.pdf"
    if not os.path.exists(file_name):
        bucket_name = "arxiv-dataset"
        blob_name = f"arxiv/arxiv/pdf/{arxiv_id.split('.')[0]}/{arxiv_id}.pdf"

        storage_client = storage.Client.create_anonymous_client()
        bucket = storage_client.bucket(bucket_name)
        blob = bucket.blob(blob_name)
        blob.download_to_filename(file_name)

        print(
            f"Downloaded {arxiv_id}.pdf from gs://{bucket_name}:{blob_name} successfully.")

    return file_name


def context(file_name) -> dict:
    context = {
        'toc': '',
        'fulltext': [],
    }

    toc_list = []
    with fitz.open(file_name) as doc:
        toc = doc.get_toc(simple=False)
        if len(toc) == 0:
            toc_list.append('No provided table of contents.')
        else:
            for entry in toc:
                toc_list.append(entry[1])

        for page in doc:
            text = page.get_text().encode("utf-8")
            text = text.replace(b'\n', b' ')
            context['fulltext'].append(text.decode("utf-8"))

    context['toc'] = "[Table Of Contents]: \n" + "\n".join(toc_list) + ";"

    return context


def remove(file_name):
    if os.path.exists(file_name):
        os.remove(file_name)

        print(f"Removed {file_name} successfully.")
